
/**
   This is an interface for a single bid for installing
   an air conditioning unit
   
   @author Charles Hoot
   @version 3.0
*/
public interface BidInterface
{
    /**
       Returns the name of the company making this bid
       Precondition: None
       Postcondition: The name was returned
       
       @return  the name of the company making the bid */
    public String getCompanyName(); 
  
    /**
       Returns the description of the air conditioner that 
       this bid is for
       Precondition: None
       Postcondition: The description was returned
       
       @return  the description of the air conditioner */
    public String getDescription();

    /**
       Returns the capacity of this bid's AC in tons (1 ton = 12,000 BTU)
       Precondition: None
       Postcondition: The performance was returned
       
       @return  the cooling capacity of the AC unit in tons */
    public double getCapacity();  
  
    /**
       Returns the seasonal efficiency of this bid's AC (SEER)
       Precondition: None
       Postcondition: The perfomance was returned
       
       @return  the efficiency of the AC unit */
    public double getSEER();
 
    /**
       Returns the cost of this bid's AC
       Precondition: None
       Postcondition: The AC cost was returned
       
       @return  the cost of the unit in dollars */
    public double getUnitCost(); 
 
    /**
       Returns the cost of installing this bid's AC
       Precondition: None
       Postcondition: The installation cost was returned
       
       @return  the cost of installation in dollars  */
    public double getInstallationCost();
 
     /**
       Returns the yearly cost of operating this bid's AC
       Precondition: None
       Postcondition: The yearly cost was returned
       
       @param hoursOperated  average number of hours the unit operates per year
       @param energyCost     cost in dollars per kilowatt hour
       @return  the cost for the year in dollars,
                cost = 12   tons   energyCost   hoursOperated / SEER */
    public double getYearlyCost(double hoursOperated, double energyCost);
}
