import java.util.Arrays;
/** Chapter 2, Project2.
	A class that implements the ADT set by using a resizable array.
	The array is never full.
	@author Frank Carrano, Karen Gardner
	@version 1.0
*/
public class ArraySet<T> implements BagInterface<T>, SetInterface<T>
{
	private T[] array; // cannot be final due to doubling
	private static final int DEFAULT_INITIAL_CAPACITY = 3; // initial capcity of array
	private int numberOfEntries;

	/** Creates an empty array whose initial capacity is 3. */
	public ArraySet()
	{
		this(DEFAULT_INITIAL_CAPACITY);
	} // end default constructor

	/** Creates an empty array having a given initial capacity.
	    @param capacity  the integer capacity desired */
	public ArraySet(int capacity)
	{
		numberOfEntries = 0;
		// the cast is safe because the new array contains null entries
		@SuppressWarnings("unchecked")
		T[] tempArray = (T[])new Object[capacity]; // unchecked cast
		array = tempArray;
	} // end constructor

	/** Creates a array containing given entries.
	    @param contents  an array of objects */
   public ArraySet(T[] contents)
   {
      array = Arrays.copyOf(contents, contents.length);
      numberOfEntries = contents.length;
   } // end constructor

	/** Adds a new entry to this array, avoiding duplicates.
	    @param newEntry  the object to be added as a new entry
	    @return true if the addition is successful, or false if not */
	public boolean add(T newEntry)
	{
		if(!contains(newEntry))
		{
			ensureCapacity();
			array[numberOfEntries] = newEntry;
			numberOfEntries++;

			return true;
		}
		return false;
	} // end add

   // Doubles the size of the array array if it is full
	private void ensureCapacity()
	{
		if (numberOfEntries == array.length)
         array = Arrays.copyOf(array, 2 * array.length);
	} // end ensureCapacity

   /** @return true; the array is never full because the array is
	   resized, as necessary */
	public boolean isFull()
	{
		return false;
	} // end isFull

	/** Retrieves all entries that are in this array.
	    @return a newly allocated array of all the entries in the array */
	public T[] toArray()
	{
		// the cast is safe because the new array contains null entries
		@SuppressWarnings("unchecked")
		T[] result = (T[])new Object[numberOfEntries]; // unchecked cast

		for (int index = 0; index < numberOfEntries; index++)
		{
			result[index] = array[index];
		} // end for

		return result;
	} // end toArray

	/** Sees whether this array is empty.
	    @return true if this array is empty, or false if not */
	public boolean isEmpty()
	{
		return numberOfEntries == 0;
	} // end isEmpty

	/** Gets the (current) capacity of this array.
	    @return the integer number of entries that the array can hold */
	public int getCapacity()
	{
		return array.length;
	} // end getCapacity

	/** Gets the number of entries currently in this array.
	    @return the integer number of entries currently in the array */
	public int getCurrentSize()
	{
		return numberOfEntries;
	} // end getCurrentSize

	/** Counts the number of times a given entry appears in this array.
		 @param anEntry  the entry to be counted
		 @return the number of times anEntry appears in the array */
	public int getFrequencyOf(T anEntry)
	{
		int counter = 0;

		for (int index = 0; index < numberOfEntries; index++)
		{
			if (anEntry.equals(array[index]))
			{
				counter++;
			} // end if
		} // end for

		return counter;
	} // end getFrequencyOf

	/** Tests whether this array contains a given entry.
		 @param anEntry  the entry to locate
		 @return true if the array contains anEntry, or false otherwise */
	public boolean contains(T anEntry)
	{
		return getIndexOf(anEntry) > -1;
	} // end contains

 	// Locates a given object within the array.
	// Returns the index of the object, if located,
	// or -1 otherwise.
	private int getIndexOf(T anEntry)
	{
		int where = -1;
		boolean found = false;

		for (int index = 0; !found && (index < numberOfEntries); index++)
		{
			if (anEntry.equals(array[index]))
			{
				found = true;
				where = index;
			} // end if
		} // end for

		return where;
	} // end getIndexOf

	/** Removes all entries from this array. */
	public void clear()
	{
		while (!isEmpty())
         remove();
	} // end clear

	/** Removes one unspecified entry from this array.
       @return either the removed entry, if the removal
               was successful, or null otherwise */
	public T remove()
	{
		T result = removeEntry(numberOfEntries - 1);

		return result;
	} // end remove

	/** Removes one occurrence of a given entry from this array.
       @param anEntry  the entry to be removed
       @return true if the removal was successful, or null otherwise */
   public boolean remove(T anEntry)
	{
		int index = getIndexOf(anEntry);
        T result = removeEntry(index);

		return anEntry.equals(result);
	} // end remove

   // Removes and returns the array entry at a given index.
   // If no such entry exists, returns null.
	private T removeEntry(int givenIndex)
	{
		T result = null;

		if (!isEmpty() && (givenIndex >= 0))
		{
         result = array[givenIndex]; // entry to remove
         numberOfEntries--;
         array[givenIndex] = array[numberOfEntries]; // replace entry to remove with last entry
         array[numberOfEntries] = null; // remove reference to last entry
		} // end if
      return result;
	} // end removeEntry
} // end ArraySet



