import java.util.Arrays;
/**
    A class of book piles whose entries are stored in an array.
    @author Frank M. Carrano
    @version 3.0
*/

public class PileOfBooks implements PileInterface
{
	private String[] pile;     // array of books
	private int topIndex; // index of top book
	private static final int DEFAULT_INITIAL_CAPACITY = 50;
	
	public PileOfBooks()
	{
      this(DEFAULT_INITIAL_CAPACITY);
 	} // end default constructor
	
	public PileOfBooks(int initialCapacity)
	{
      pile = new String[initialCapacity];
		topIndex = -1;
	} // end constructor

	public void add(String newBook)
	{
		ensureCapacity();
		topIndex++;
		pile[topIndex] = newBook;
	} // end add

   // Doubles the size of the array pile if it is full
	private void ensureCapacity() 
	{
 	   if (topIndex == pile.length - 1) // if array is full,
         // double size of array
         pile = Arrays.copyOf(pile, 2 * pile.length);
	} // end ensureCapacity

	public String getTopBook()
	{
		String top = null;
		
		if (!isEmpty())
			top = pile[topIndex];
		
		return top;
	} // end getTopBook

	public String remove()
	{
		String top = null;
		
		if (!isEmpty())
		{
			top = pile[topIndex];
			pile[topIndex] = null;
			topIndex--; 
		} // end if
		
		return top;
	} // end remove

   public boolean isEmpty()
	{
		return topIndex < 0;
	} // end isEmpty
	
	public void clear()
	{
		for (; topIndex > -1; topIndex--)
			pile[topIndex] = null;
   //	Assertion: topIndex is -1	
	} // end clear
} // end PileOfBooks