/**
   This is an interface for a collection of objects that are 
   arranged in a ring. One object is marked as the current one, and you can advance 
   this marker from one object to the next one.
   @author Charles Hoot
   @author Frank M. Carrano
   @version 3.0
*/
public interface RingInterface<T>
{
    /**
       Adds an item to this ring.
       Precondition: None.
       Postcondition: The item was added at the end of the ring.
       @param  toAdd  the item to add. */
    public void add(T toAdd);
    
    /**
       Removes and returns the entry at the beginning of this ring.
       Precondition: None.
       Postcondition: The first entry in the ring was removed
                      from the ring and returned, if the ring was not empty.
       @return the first entry in the ring or null, if the ring is empty */
    public T remove();
    
    /**
       Removes all of the entries from this ring.
       Precondition: None.
       Postcondition: The ring is empty. */
    public void clear();
    
    /**
       Returns the current entry in this ring.
       Precondition: None.
       Postcondition: The current entry in the ring was returned, if the ring is not empty
       @return the current entry in the ring or null if the ring is empty */
    public T getCurrentEntry();
    
    /**
       Advances the marker of the current entry in this ring. 
       Precondition: The ring is not empty.
       Postcondition: The marker is advanced by one entry. */
    public void advance();
    
    /**
       Gets the number of entries in this ring.
       Precondition: None.
       Postcondition: The ring is unchanged.
       @return the number of entries in the ring */
    public int getLength();
    
    /**
       Sees whether this ring is empty. 
       Precondition: None.
       Postcondition: The ring is unchanged.
       @return true if there are no items in the ring, false otherwise */
    public boolean isEmpty();
}
