import java.io.File;
import java.io.FileNotFoundException;
import java.util.Scanner;
/** A spelling checker.
    @author Frank M. Carrano
    @version 3.0
*/
public class SpellChecker
{
   BagInterface<String> correctWords;
   
   public SpellChecker()
   {
      correctWords = new ResizableArrayBag<String>();
   } // end default constructor
   
   /** Initializes the dictionary of correctly spelled words. 
       @param correctWords  a File object that represents a text file 
                            of correctly spelled words, one per line
       @return true if the dictionary is initialized, otherwise returns false */
	public boolean setDictionary(File correctWordFile)
	{
      boolean fileOpened = true;
      boolean continueAdding = true;
      try
      {
         Scanner fileData = new Scanner(correctWordFile);
         while (continueAdding && fileData.hasNextLine())
         {
            String word = fileData.nextLine();
            continueAdding = correctWords.add(word);
         } // end while
         fileData.close();
      }
      catch (FileNotFoundException e)
      {
         fileOpened = false; // error opening the file
      }
      
      return fileOpened && continueAdding;
   } // end setDictionary
   
   /** Checks the spelling of a given bag of words. 
       @param wordBag  a bag of words whose spelling is to be checked
       @return a bag of the words in wordBag whose spelling is incorrect */
	public BagInterface<String> checkSpelling(BagInterface<String> wordBag)
	{
      // do not destroy the given bag of words
      Object[] wordArray = wordBag.toArray();
      int wordCount = wordBag.getCurrentSize();
      BagInterface<String> tempWordBag = new ResizableArrayBag<String>();
		for (int index = 0; index < wordCount; index++) 
         tempWordBag.add((String)wordArray[index]);

      return tempWordBag.difference(correctWords);
   } // end checkSpelling
} // end SpellChecker
